/*
 * This file is part of presence  *
 *
 * Copyright  2011 Nokia Corporation and/or its subsidiary(-ies). Allrights reserved.
 *
 * Contact: Alvi Hirvela <Alvie.Hirvela@nokia.com>
 *
 * This software, including documentation, is protected by copyrightcontrolled by
 * Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
 * adapting or translating, any or all of this material requires the prior written consent
 * of Nokia Corporation. This material also contains confidential information which may
 * not be disclosed to others  without the prior written consent of Nokia.
 */
#ifndef SINGLETONBASE_H
#define SINGLETONBASE_H



/**
  \ingroup libpresence
\class SingletonBase
\brief Implements a base interface for singleton classes as template class
        Usage: Simply derive your singleton class from this base classe by adding:
           class MyClass : public SingletonBase<MyClass>

           <p>
           To prevent from linker errors be sure to set the singleton variable m_pInstance in your cpp file as follows:<br>
            \code
           template<> MyClass* SingletonBase<MyClass>::m_pInstance = NULL;
           \endcode
           </p>

*/

#include <MExport>

template<class SINGLETON_CLASS> class SingletonBase
{
private:

    SingletonBase( const SingletonBase<SINGLETON_CLASS>&) {};


protected:

    SingletonBase(void) {};

    virtual ~SingletonBase(){};


public:

    static SINGLETON_CLASS* instance()
    {
        if ( m_pInstance == 0 ) {
            // first time call of function --> create instance
            m_pInstance = new SINGLETON_CLASS;
        }

        return m_pInstance;
    }


    static void killInstance()
    {
        if ( m_pInstance != 0 ) {
            // clean up the singleton instance
            delete m_pInstance;
            m_pInstance = 0;
        }
    }


protected:

    /** \brief Pointer to singleton instance
    \remark be sure to define this variable in your derived class's .cpp file as follows: <br>
    <b>template<> MyClass* SingletonBase<MyClass>::m_pInstance = NULL;</b>
    */
    static SINGLETON_CLASS* m_pInstance;
};

#endif // SINGLETONBASE_H
